package com.fzm.common.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fzm.common.constant.SystemConstant;
import com.fzm.common.entity.EntrustShelf;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.mapper.EntrustShelfMapper;
import com.fzm.common.service.EntrustShelfService;
import com.fzm.common.service.NftService;
import com.fzm.common.utils.JwtUtil;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

/**
 * @author tangtuo
 * @date 2021/7/5 15:09
 */
@Service
@Transactional(rollbackFor = RuntimeException.class)
public class EntrustShelfServiceImpl extends ServiceImpl<EntrustShelfMapper, EntrustShelf> implements EntrustShelfService {

    @Resource
    private HttpServletRequest request;

    @Resource
    private NftService nftService;

    @Override
    public Boolean submit(EntrustShelf entrustShelf) {
        // 修改当前nft的是否已申请的状态为是
        nftService.updateEntrust(entrustShelf.getNftHash(), SystemConstant.BOOLEAN_DATA_TRUE);
        String authorization = request.getHeader("Authorization");
        Integer userId = JwtUtil.getUserIdFromToken(authorization);
        entrustShelf.setUserId(userId);
        try {
            return this.save(entrustShelf);
        } catch (DuplicateKeyException e) {
            throw GlobalException.newException(ResultCode.FAILED, "此nft已提交过上架申请,请勿重复提交");
        }
    }

    @Override
    public Boolean cancel(Integer id) {
        EntrustShelf entrustShelf = this.getById(id);
        if (entrustShelf == null) {
            throw GlobalException.newException(ResultCode.SELECT_FAILED);
        }
        // 已取消
        if (entrustShelf.getStatus().equals(SystemConstant.BOOLEAN_DATA_TRUE)) {
            throw GlobalException.newException(ResultCode.FAILED, "当前申请已取消");
        }
        // 修改当前nft的是否已申请的状态为否
        nftService.updateEntrust(entrustShelf.getNftHash(), SystemConstant.BOOLEAN_DATA_FALSE);
        entrustShelf.setStatus(SystemConstant.BOOLEAN_DATA_TRUE);
        return this.updateById(entrustShelf);
    }
}
